<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;

use Teknisa\GER\Modules\Cardapio\Controllers\TeknisaCrud;
use Teknisa\GER\Utilities\Environment;
use Doctrine\ORM\EntityManager;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\FilterCriteria;


class Pla06800LiberacaoDeCardapioParaAgregadasService {
    const GET_ALL_SERVICES = "pla06800_getServico";
    
    public function __construct(Environment $environment, $entityManager, $dataSourceManager, $novoCodigo) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->novoCodigo = $novoCodigo;
    }
    
    public function getAllServices($cdFilial) {
        $sql    = "SELECT S.CDSERVFILI, S.NMSERVICO FROM FILIAL F, SERVICO S WHERE F.CDFILIAL = :P_CDFILIAL AND F.IDFILCENTPME = 'S' AND F.CDFILIAL = S.CDFILIAL  AND IDTIPOSERV = 'N' AND EXISTS (SELECT 1 FROM FILIOPER FO WHERE FO.CDFILIAL = F.CDFILIAL AND FO.CDOPERADOR = :CDOPERADOR) ORDER BY S.CDSERVFILI";
        $params = array(
            "P_CDFILIAL"    => $cdFilial,
            "CDOPERADOR"  => $this->environment->getCdOperador()
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }  
    
    public function getCentralEAgregadas($cdFilial) {
        $sql    = "SELECT CDFILIAL FROM FILIAL WHERE CDFILIAL = :P_CDFILIAL OR CDFILCENTPME = :P_CDFILIAL";
        $params = array(
            "P_CDFILIAL" => $cdFilial
        );    
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();        
    }
    
    public function updateCardaMes($row, $unidade, $servico) {
        $sql    = "UPDATE CARDAMES SET IDCARDLIBCENT = 'N', IDCARDLIBAGRE = 'N' WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND DTMESCARDAPIO = :P_DTMES";
        $params = array(
            "P_CDFILIAL"    => $unidade,
            "P_CDSERVFILI"  => $servico,
            "P_DTMES"       => $row["FORMATEDDTCARDAPIO"]
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $query->execute($params);
    }
    
    public function deleteAprovCard($row, $unidade, $servico) {
        $sql    = "DELETE FROM CTRLAPROVCARD WHERE (CDFILIAL = :P_CDFILIAL) AND (CDSERVFILI = :P_CDSERVFILI) AND (DTMESCARDAPROV = :P_DTMESCARD)";
        $params = array(
            "P_CDFILIAL"    => $unidade,
            "P_CDSERVFILI"  => $servico,
            "P_DTMESCARD"   => $row["FORMATEDDTCARDAPIO"]
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $query->execute($params);    
    }
    
    public function procedureConsumoPlanejado($row) {
        $row['CDOPERADOR'] = $this->environment->getCdOperador();
        $row['P_DTINICIAL'] = $row['FORMATEDDTCARDAPIO'];
        $row['P_DTFINAL'] = $row['FORMATEDDTCARDAPIOEND'];
        $cdServFili = ($row['CDSERVFILI'] == "" || count($row['CDSERVFILI']) == 0) ? "T" : implode("_", $row['CDSERVFILI']);
        $nrconsplan = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $this->environment->getNrOrg());
        $row["NRCONSPLAN"] = $nrconsplan;
        
        $connection = $this->entityManager->getConnection();
        $procedureConsumoPlanejado = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PLA');
        $procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $row['CDFILIAL'],          Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, $cdServFili,               Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['P_DTINICIAL'],       Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['P_DTFINAL'],         Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "C",                       Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, "CFP",                     Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, "N",                       Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                         Param::PARAM_TYPE_INT));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",                       Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $nrconsplan,               Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, "0",                       Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, "zzzzzzzzzz",              Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'],        Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "",                        Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "",                        Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->execute();
    }
    
    public function retornaMediaEMeta($row, $servico) {
        $sql = "SELECT CP.CDFILICPLA, CP.CDSERVCPLA, C.VRCUSTPADR AS META, CP.MEDIA FROM (SELECT CDFILICPLA, CDSERVCPLA, CD.CDFILIAL, CD.CDSERVFILI, CD.DTREFECUSTMES, CD.TOTCOMENS, SUM(DECODE(CD.TOTCOMENS, 0, 0, NULL, 0, (CUSTO_PRODUTO(CDFILICPLA, CDPRODCPLA, :P_TIPOCUSTO, CD.DTREFECUSTMES, SYSDATE) * QTCONSPLAN) / CD.TOTCOMENS)) AS MEDIA FROM CONSPLAN, (SELECT C.CDFILIAL, C.CDSERVFILI, NVL(CM.DTREFECUSTMES, SYSDATE) AS DTREFECUSTMES, SUM(DECODE(P.IDCOMCUSPERC, 'T', C.QTCOMESTICAR, C.QTCLIESTICAR)) AS TOTCOMENS FROM CARDADIA C, PARAMFILIAL P, CARDAMES CM WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI AND C.DTMESCARD = :P_DTMES AND C.CDFILIAL = P.CDFILIAL AND C.CDFILIAL = CM.CDFILIAL AND C.CDSERVFILI = CM.CDSERVFILI AND C.DTMESCARD = CM.DTMESCARDAPIO GROUP BY C.CDFILIAL, C.CDSERVFILI, CM.DTREFECUSTMES) CD WHERE NRCONSPLAN = :P_NRCONSPLAN AND CDFILICPLA = :P_CDFILIAL AND CDFILICPLA = CD.CDFILIAL AND CDSERVCPLA = CD.CDSERVFILI GROUP BY CDFILICPLA, CDSERVCPLA, CD.CDFILIAL, CD.CDSERVFILI, CD.DTREFECUSTMES, CD.TOTCOMENS) CP, CUSTPADR C WHERE C.DTREFCUSPADR(+) = :P_DTMES AND CP.CDFILICPLA = C.CDFILIAL(+) AND CP.CDSERVCPLA = C.CDSERVFILI(+)";
        
        $params = array(
            "P_CDFILIAL"    => $row["CDFILIAL"],
            "P_TIPOCUSTO"   => $row["CDTIPOCUSTO"],
            "P_CDSERVFILI"  => $servico,
            "P_NRCONSPLAN"  => $row["NRCONSPLAN"],
            "P_DTMES"       => $row["FORMATEDDTCARDAPIO"]
        );    
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();          
    }
    
    public function procedureVerificaCardPadrao($params) {
        $connection = $this->entityManager->getConnection();
        $procedureVerificaCardPadrao = new StoredProcedure($connection, 'VERIFICA_CARD_PADRAO');
        $procedureVerificaCardPadrao->addParam(new Param('P_FILIAL',     Param::PARAM_INPUT, $params['P_CDFILIAL'],         Param::PARAM_TYPE_STR));
        $procedureVerificaCardPadrao->addParam(new Param('P_DTMESCARD',  Param::PARAM_INPUT, $params['P_DTMESCARD'],        Param::PARAM_TYPE_STR));
        $procedureVerificaCardPadrao->addParam(new Param('P_NRLISTA',    Param::PARAM_INPUT, $params['P_NRLISTA'],          Param::PARAM_TYPE_STR));        
        $procedureVerificaCardPadrao->execute();
    }
    
    public function retornaPratosInconsistentes($row) {
        $sql = "SELECT L.CDFILIAGRE|| ' ' || FA.NMFILIAL AS FILIAGRE, L.CDSERVFILI || ' ' || SE.NMSERVICO AS SERVICO, L.DTMESCARD, L.CDPRATO || ' ' || P.NMPRATO as PRATO, L.IDLOGCARDPAD, L.CDPRATO, P.NMPRATO FROM LOGCARDPADRAO L, PRATO P, FILIAL FC, FILIAL FA, SERVICO SE WHERE L.CDFILIAL = :P_CDCENTRAL AND L.DTMESCARD = :P_DTMESCARD AND P.CDPRATO = L.CDPRATO AND P.CDFILAUXPRAT = L.CDFILAUXPRAT AND FC.CDFILIAL = L.CDFILIAL AND FA.CDFILIAL = L.CDFILIAGRE AND SE.CDFILIAL = L.CDFILIAL AND SE.CDSERVFILI = L.CDSERVFILI ORDER BY L.IDLOGCARDPAD, L.CDFILIAGRE, L.CDPRATO";
        $params = [
            "P_CDCENTRAL" => $row["P_CDFILIAL"],
            "P_DTMESCARD" => $row["P_DTMESCARD"]
        ];
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $query->execute($params);
        return $query->fetchAll(); 
    }
    
    public function procedureLiberaCardPadrao($paramslibeCard) {
        $connection = $this->entityManager->getConnection();
        $procedureLiberaCardapioPadrao = new StoredProcedure($connection, 'LIBERA_CARD_PADRAO');
        $procedureLiberaCardapioPadrao->addParam(new Param('P_FILIAL',     Param::PARAM_INPUT, $paramslibeCard['P_CDFILIAL'],         Param::PARAM_TYPE_STR));
        $procedureLiberaCardapioPadrao->addParam(new Param('P_DTMESCARD',  Param::PARAM_INPUT, $paramslibeCard['P_DTMESCARD'],        Param::PARAM_TYPE_STR));
        $procedureLiberaCardapioPadrao->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $paramslibeCard['P_CDOPERADOR'],       Param::PARAM_TYPE_STR));
        $procedureLiberaCardapioPadrao->addParam(new Param('P_NRLISTA',    Param::PARAM_INPUT, $paramslibeCard['P_NRLISTA'],          Param::PARAM_TYPE_STR));        
        $procedureLiberaCardapioPadrao->execute();        
    }
    
    public function retornCustPadr($row) {
        $sql = "SELECT VRCUSTPADR FROM CUSTPADR WHERE CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND DTREFCUSPADR = :P_DTREFCUSPADR";
        
        $params = [
            "P_CDFILIAL"        => $row["CDFILIAL"],
            "P_CDSERVFILI"      => $row["CDSERVFILI"],
            "P_DTREFCUSPADR"    => $row["DATA_INICIAL"]
        ];
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $query->execute($params);
        return $query->fetchAll();         
    }

}

